using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using System.Net;

namespace SeriousTFSCustomization.WinUI
{
	public partial class Form1 : Form
	{
		private TeamFoundationServer m_server;
		private WorkItemStore m_store;
		
		public Form1()
		{
			InitializeComponent();

			ConnectToTeamFoundationServer();
			PopulateWorkItemQueryList();
		}

		private void PopulateWorkItemQueryList()
		{
			foreach (Project project in m_store.Projects)
			{
				foreach (StoredQuery query in project.StoredQueries)
				{
					m_listQueries.Items.Add(
						new StoredQueryWrapper(query));
				}
			}
		}

		public class StoredQueryWrapper
		{
			/// <summary>
			/// Initializes a new instance of the StoredQueryWrapper class.
			/// </summary>
			/// <param name="query"></param>
			public StoredQueryWrapper(StoredQuery query)
			{
				m_query = query;
			}

			private StoredQuery m_query;
			public StoredQuery Query
			{
				get
				{
					return m_query;
				}
				set
				{
					m_query = value;
				}
			}

			public override string ToString()
			{
				return String.Format(
					"{0} - {1}", 
					this.Query.Project.Name, this.Query.Name);
			}
		}

		private void ConnectToTeamFoundationServer()
		{
			try
			{
				if (m_server == null)
				{
					m_server = new TeamFoundationServer(
						"http://labtfs:8080");

					m_server.Authenticate();
					m_status.Text = "Connected";

					m_store = m_server.GetService(
						typeof(WorkItemStore)) as WorkItemStore;
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			} 			
		}

		private void m_listQueries_SelectedIndexChanged(object sender, EventArgs e)
		{
			try
			{
				PopulateQuery();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			} 
		}

		private string m_currentProjectName;

		private void PopulateQuery()
		{
			StoredQueryWrapper queryWrapper = 
				m_listQueries.SelectedItem as StoredQueryWrapper;

			if (queryWrapper != null)
			{
				m_textQuery.Text = 
					FormatQuery(queryWrapper.Query.QueryText);
//				m_textQuery.Text =
//					queryWrapper.Query.QueryText;

				m_currentProjectName = 
					queryWrapper.Query.Project.Name;
			}
		}

		private string FormatQuery(string query)
		{
			StringBuilder builder = new StringBuilder(query.Trim());

			builder.Replace("SELECT ", String.Format("SELECT {0}", Environment.NewLine));
			builder.Replace("FROM ", String.Format("{0}FROM {0}", Environment.NewLine));
			builder.Replace("WHERE ", String.Format("{0}WHERE {0}", Environment.NewLine));
			builder.Replace("ORDER BY ", String.Format("{0}ORDER BY {0}", Environment.NewLine));
			builder.Replace("GROUP BY ", String.Format("{0}GROUP BY {0}", Environment.NewLine));
			builder.Replace("], ", String.Format("], {0}", Environment.NewLine));

			return builder.ToString();
		}

		private void runQueryToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				RunQuery();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			} 
		}

		private void RunQuery()
		{
			RunQuery(m_textQuery.Text);
		}

		private void RunQuery(string wiqlQuery)
		{
			m_status.Text = "Running query...";

			Dictionary<string, string> values = 
				new Dictionary<string, string>();

			values.Add("project", m_currentProjectName);

			Query query = new Query(m_store, wiqlQuery, values);

			m_grid.LoadFromWiql(m_store, query.QueryString);

			m_status.Text = "Query executed.";
		}

		private WorkItemDetailForm m_currentWorkItemDetailForm;

		private WorkItem GetSelectedWorkItem()
		{
			if (m_grid.SelectedRows == null || m_grid.SelectedRows.Count == 0)
			{
				return null;
			}
			else
			{
				int index = m_grid.SelectedRows[0].Index;

				WorkItem item = m_grid.WorkItemCollection[index];

				if (m_currentWorkItemDetailForm == null ||
					m_currentWorkItemDetailForm.IsDisposed == true)
				{
					m_currentWorkItemDetailForm = new WorkItemDetailForm();
				}
				return item;
			}			
		}

		private void m_grid_SelectionChanged(object sender, EventArgs e)
		{
			try
			{
				if (m_grid.SelectedRows != null && 
					m_grid.SelectedRows.Count > 0)
				{
					WorkItem item = GetSelectedWorkItem();
					m_currentWorkItemDetailForm.WorkItem = item;

					m_currentWorkItemDetailForm.Show();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private LinkEditor m_linkEditor;
		public LinkEditor LinkEditor
		{
			get
			{
				if (m_linkEditor == null) m_linkEditor = new LinkEditor();

				return m_linkEditor;
			}
		}

		private void linkFromToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (GetSelectedWorkItem() != null)
				{
					this.LinkEditor.FromWorkItem = GetSelectedWorkItem();
					this.LinkEditor.Show();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			} 
		}

		private void linkToToolStripMenuItem_Click(object sender, EventArgs e)
		{
			try
			{
				if (GetSelectedWorkItem() != null)
				{
					this.LinkEditor.AddLinkTargetWorkItem(GetSelectedWorkItem());
					this.LinkEditor.Show();
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			} 
		}

	}
}